@@ -15,15 +15,17 @@
 Low-level interactions with the D-Bus.
 """
 
+# isort: THIRDPARTY
 import dbus
 
 from ._constants import SERVICE
 
 
-class Bus():
+class Bus:
     """
     Our bus.
     """
+
     # pylint: disable=too-few-public-methods
 
     _BUS = None
