@@ -25,7 +25,7 @@ from uuid import UUID
 # isort: THIRDPARTY
 from justbytes import Range
 
-from .._constants import YesOrNo
+from .._constants import PoolIdType, YesOrNo
 from .._error_codes import PoolErrorCode
 from .._errors import (
     StratisCliEngineError,
@@ -41,12 +41,12 @@ from .._errors import (
     StratisCliPartialChangeError,
     StratisCliResourceNotFoundError,
 )
-from .._stratisd_constants import BlockDevTiers, PoolIdType, StratisdErrors
+from .._stratisd_constants import BlockDevTiers, StratisdErrors
 from ._connection import get_object
 from ._constants import TOP_OBJECT
 from ._formatting import get_property, get_uuid_formatter
 from ._list_pool import list_pools
-from ._utils import get_clevis_info
+from ._utils import ClevisInfo, PoolSelector
 
 
 def _generate_pools_to_blockdevs(managed_objects, to_be_added, tier):
@@ -176,7 +176,7 @@ class PoolActions:
 
         _check_same_tier(pool_name, managed_objects, blockdevs, BlockDevTiers.DATA)
 
-        clevis_info = get_clevis_info(namespace)
+        clevis_info = ClevisInfo.get_info_from_namespace(namespace)
 
         (
             (changed, (pool_object_path, _)),
@@ -221,28 +221,29 @@ class PoolActions:
         :raises StratisCliEngineError:
         """
         # pylint: disable=import-outside-toplevel
-        from ._data import Manager, ObjectManager, pools
+        from ._data import Manager
 
         proxy = get_object(TOP_OBJECT)
-        managed_objects = ObjectManager.Methods.GetManagedObjects(proxy, {})
-        pool_name = namespace.pool_name
-        (pool_object_path, _) = next(
-            pools(props={"Name": pool_name})
-            .require_unique_match(True)
-            .search(managed_objects)
+
+        (pool_id, id_type) = (
+            (namespace.uuid.hex, "uuid")
+            if getattr(namespace, "name") is None
+            else (namespace.name, "name")
         )
 
         ((stopped, _), return_code, message) = Manager.Methods.StopPool(
-            proxy, {"pool": pool_object_path}
+            proxy,
+            {
+                "id": pool_id,
+                "id_type": id_type,
+            },
         )
 
         if return_code != StratisdErrors.OK:  # pragma: no cover
             raise StratisCliEngineError(return_code, message)
 
-        if not stopped:  # pragma: no cover
-            raise StratisCliIncoherenceError(
-                f"Expected to stop pool with name {pool_name} but it was already stopped."
-            )
+        if not stopped:
+            raise StratisCliNoChangeError("stop", pool_id)
 
     @staticmethod
     def start_pool(namespace):
@@ -258,16 +259,16 @@ class PoolActions:
         proxy = get_object(TOP_OBJECT)
 
         (pool_id, id_type) = (
-            (namespace.uuid.hex, PoolIdType.UUID)
+            (namespace.uuid.hex, "uuid")
             if getattr(namespace, "name") is None
-            else (namespace.name, PoolIdType.NAME)
+            else (namespace.name, "name")
         )
 
         ((started, _), return_code, message) = Manager.Methods.StartPool(
             proxy,
             {
                 "id": pool_id,
-                "id_type": str(id_type),
+                "id_type": id_type,
                 "unlock_method": (False, "")
                 if namespace.unlock_method is None
                 else (True, namespace.unlock_method),
@@ -351,9 +352,9 @@ class PoolActions:
         uuid_formatter = get_uuid_formatter(namespace.unhyphenated_uuids)
 
         selection = (
-            (None if pool_name is None else (PoolIdType.NAME, pool_name))
+            (None if pool_name is None else PoolSelector(PoolIdType.NAME, pool_name))
             if pool_uuid is None
-            else (PoolIdType.UUID, pool_uuid)
+            else PoolSelector(PoolIdType.UUID, pool_uuid)
         )
 
         return list_pools(uuid_formatter, stopped=stopped, selection=selection)
@@ -579,44 +580,16 @@ class PoolActions:
                 else new_size > Range(modev.TotalPhysicalSize())
             )
 
-        if namespace.device_uuid == []:
-            expand_modevs = [modev for modev in modevs if expandable(modev)]
+        def expand(pool_proxy, modev):  # pragma: no cover
+            """
+            Expand a pool by extending exactly one expandable device in the
+            pool.
 
-        else:
-            device_uuids = frozenset(uuid.hex for uuid in namespace.device_uuid)
-            expand_modevs = [modev for modev in modevs if modev.Uuid() in device_uuids]
-
-            if len(expand_modevs) < len(device_uuids):
-                missing_uuids = device_uuids.difference(
-                    frozenset(UUID(modev.Uuid()) for modev in expand_modevs)
-                )
-
-                missing_uuids = ", ".join(str(UUID(uuid)) for uuid in missing_uuids)
-
-                raise StratisCliResourceNotFoundError(
-                    "extend-data", f"devices with UUIDs {missing_uuids}"
-                )
-
-            t_1, t_2 = tee(expand_modevs)
-            expandable_modevs, unexpandable_modevs = (
-                [modev for modev in t_1 if expandable(modev)],
-                [modev for modev in t_2 if not expandable(modev)],
-            )
-
-            if unexpandable_modevs != []:  # pragma: no cover
-                raise StratisCliPartialChangeError(
-                    "extend-data",
-                    frozenset(str(UUID(modev.Uuid())) for modev in expandable_modevs),
-                    frozenset(str(UUID(modev.Uuid())) for modev in unexpandable_modevs),
-                )
-
-        assert isinstance(expand_modevs, list)
-        if expand_modevs == []:
-            raise StratisCliNoDeviceSizeChangeError()
-
-        for modev in expand_modevs:  # pragma: no cover
+            :param pool_proxy: dbus proxy object for the pool
+            :param MoDev modev: represents D-Bus information on a single device
+            """
             (changed, return_code, message) = Pool.Methods.GrowPhysicalDevice(
-                get_object(pool_object_path), {"dev": modev.Uuid()}
+                pool_proxy, {"dev": modev.Uuid()}
             )
 
             if return_code != StratisdErrors.OK:
@@ -631,6 +604,62 @@ class PoolActions:
                     )
                 )
 
+        def get_devices_to_expand(device_uuids, modevs):
+            """
+            Calculate devices to expand
+            :param device_uuids: a list of device uuids, if empty expand all
+            :type device_uuids: list of UUID
+            :param modevs: MODev objects representing all devices in the pool
+            :type modevs: list of MODev
+            :return: list of MODev objects representing devices to expand
+            """
+            if device_uuids == []:
+                expand_modevs = [modev for modev in modevs if expandable(modev)]
+            else:
+                device_uuids = frozenset(uuid.hex for uuid in device_uuids)
+                expand_modevs = [
+                    modev for modev in modevs if modev.Uuid() in device_uuids
+                ]
+                if len(expand_modevs) < len(device_uuids):
+                    missing_uuids = device_uuids.difference(
+                        frozenset(UUID(modev.Uuid()) for modev in expand_modevs)
+                    )
+
+                    missing_uuids = ", ".join(str(UUID(uuid)) for uuid in missing_uuids)
+
+                    raise StratisCliResourceNotFoundError(
+                        "extend-data", f"devices with UUIDs {missing_uuids}"
+                    )
+
+                t_1, t_2 = tee(expand_modevs)
+                expandable_modevs, unexpandable_modevs = (
+                    [modev for modev in t_1 if expandable(modev)],
+                    [modev for modev in t_2 if not expandable(modev)],
+                )
+
+                if unexpandable_modevs != []:  # pragma: no cover
+                    raise StratisCliPartialChangeError(
+                        "extend-data",
+                        frozenset(
+                            str(UUID(modev.Uuid())) for modev in expandable_modevs
+                        ),
+                        frozenset(
+                            str(UUID(modev.Uuid())) for modev in unexpandable_modevs
+                        ),
+                    )
+
+            return expand_modevs
+
+        expand_modevs = get_devices_to_expand(namespace.device_uuid, modevs)
+
+        assert isinstance(expand_modevs, list)
+        if expand_modevs == []:
+            raise StratisCliNoDeviceSizeChangeError()
+
+        pool_proxy = get_object(pool_object_path)  # pragma: no cover
+        for modev in expand_modevs:  # pragma: no cover
+            expand(pool_proxy, modev)
+
     @staticmethod
     def set_fs_limit(namespace):
         """
@@ -660,7 +689,7 @@ class PoolActions:
         # pylint: disable=import-outside-toplevel
         from ._data import MOPool, ObjectManager, Pool, pools
 
-        decision = bool(YesOrNo.from_str(namespace.decision))
+        decision = bool(YesOrNo(namespace.decision))
 
         proxy = get_object(TOP_OBJECT)
         managed_objects = ObjectManager.Methods.GetManagedObjects(proxy, {})
