@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """
-Error heirarchy for stratis cli.
+Error hierarchy for stratis cli.
 """
 
 from ._stratisd_constants import BlockDevTiers, StratisdErrors
@@ -83,7 +83,7 @@ class StratisCliOverprovisionChangeError(StratisCliNoPropertyChangeError):
 
 class StratisCliFsLimitChangeError(StratisCliNoPropertyChangeError):
     """
-    Raised when the user requests the same FsLimti value that the pool already
+    Raised when the user requests the same FsLimit value that the pool already
     has.
     """
 
@@ -91,9 +91,21 @@ class StratisCliFsLimitChangeError(StratisCliNoPropertyChangeError):
         return f"Pool's filesystem limit is exactly {str(self.value).lower()}"
 
 
+class StratisCliFsSizeLimitChangeError(StratisCliNoPropertyChangeError):
+    """
+    Raised when the user requests the same filesystems SizeLimit value that the
+    filesystem already has.
+    """
+
+    def __str__(self):
+        if self.value is None:
+            return "Filesystem's size limit is not set"
+        return f"Filesystem's size limit is exactly {self.value}"
+
+
 class StratisCliHasCacheChangeError(StratisCliNoPropertyChangeError):
     """
-    Raised when the user request cache initialzation, but a cache is already
+    Raised when the user request cache initialization, but a cache is already
     present. It is not currently possible to remove a cache.
     """
 
@@ -424,8 +436,8 @@ class StratisCliEngineError(StratisCliRuntimeError):
         """
         # pylint: disable=super-init-not-called
         try:
-            self.error_code = StratisdErrors.from_int(rc)
-        except StopIteration:
+            self.error_code = StratisdErrors(rc)
+        except ValueError:
             self.error_code = None
 
         self.message = message
@@ -500,3 +512,13 @@ class StratisCliStratisdVersionError(StratisCliRuntimeError):
             self.minimum_version,
             self.maximum_version,
         )
+
+
+class StratisCliPassphraseMismatchError(StratisCliUserError):
+    """
+    Raised if the user specified passphrase did not match
+    the verified passphrase.
+    """
+
+    def __str__(self):
+        return "Passphrases do not match"
